@@ -4,6 +4,7 @@ from .custom_library import CustomLibrary
 from .fourier_library import FourierLibrary
 from .generalized_library import GeneralizedLibrary
 from .identity_library import IdentityLibrary
+from .parameterized_library import ParameterizedLibrary
 from .pde_library import PDELibrary
 from .polynomial_library import PolynomialLibrary
 from .sindy_pi_library import SINDyPILibrary
@@ -20,4 +21,5 @@ __all__ = [
     "PDELibrary",
     "WeakPDELibrary",
     "SINDyPILibrary",
+    "ParameterizedLibrary",
 ]
