@@ -341,7 +341,7 @@ class TrappingSR3(SR3):
             for j in range(r):
                 for k in range(r):
                     for kk in range(N):
-                        if i == kk and j == k:
+                        if i == k and j == kk:
                             PL_tensor_unsym[i, j, k, kk] = 1.0
 
         # Now symmetrize PL
@@ -478,18 +478,13 @@ class TrappingSR3(SR3):
 
     def _objective(self, x, y, coef_sparse, A, PW, q):
         """Objective function"""
-        if q != 0:
-            print_ind = q % (self.max_iter // 10.0)
-        else:
-            print_ind = q
-
         # Compute the errors
         R2 = (y - np.dot(x, coef_sparse)) ** 2
         A2 = (A - PW) ** 2
         L1 = self.threshold * np.sum(np.abs(coef_sparse.flatten()))
 
         # convoluted way to print every max_iter / 10 iterations
-        if print_ind == 0 and self.verbose:
+        if self.verbose and q % max(1, self.max_iter // 10) == 0:
             row = [
                 q,
                 0.5 * np.sum(R2),
