@@ -2,7 +2,6 @@ from itertools import repeat
 from typing import Sequence
 
 import numpy as np
-from numpy.random import choice
 from scipy.optimize import bisect
 from sklearn.base import MultiOutputMixin
 from sklearn.utils.validation import check_array
@@ -15,11 +14,27 @@ from sklearn.utils.validation import check_array
 T_DEFAULT = object()
 
 
+def flatten_2d_tall(x):
+    return x.reshape(x.size // x.shape[-1], x.shape[-1])
+
+
 def validate_input(x, t=T_DEFAULT):
+    """Forces input data to have compatible dimensions, if possible.
+
+    Args:
+        x: array of input data (measured coordinates across time)
+        t: time values for measurements.
+
+    Returns:
+        x as 2D array, with time dimension on first axis and coordinate
+        index on second axis.
+    """
     if not isinstance(x, np.ndarray):
         raise ValueError("x must be array-like")
     elif x.ndim == 1:
         x = x.reshape(-1, 1)
+    x_new = flatten_2d_tall(x)
+
     check_array(x, ensure_2d=False, allow_nd=True)
 
     if t is not T_DEFAULT:
@@ -38,181 +53,83 @@ def validate_input(x, t=T_DEFAULT):
         else:
             raise ValueError("t must be a scalar or array-like.")
 
-    if x.ndim != 2:
-        x_new = x.reshape(x.size // x.shape[-1], x.shape[-1])
-    else:
-        x_new = x
     return x_new
 
 
-def validate_control_variables(
-    x, u, multiple_trajectories=False, trim_last_point=False, return_array=True
-):
+def validate_no_reshape(x, t=T_DEFAULT):
+    """Check types and numerical sensibility of arguments.
+
+    Args:
+        x: array of input data (measured coordinates across time)
+        t: time values for measurements.
+
+    Returns:
+        x as 2D array, with time dimension on first axis and coordinate
+        index on second axis.
     """
-    Ensure that control variables u are compatible with the data x.
-    If ``return_array`` and ``multiple_trajectories`` are True, convert u from a list
-    into an array (of concatenated list entries).
+    if not isinstance(x, np.ndarray):
+        raise TypeError("Input value must be array-like")
+    check_array(x, ensure_2d=False, allow_nd=True)
+
+    if t is not T_DEFAULT:
+        if t is None:
+            raise ValueError("t must be a scalar or array-like.")
+        # Apply this check if t is a scalar
+        elif np.ndim(t) == 0 and (isinstance(t, int) or isinstance(t, float)):
+            if t <= 0:
+                raise ValueError("t must be positive")
+        # Only apply these tests if t is array-like
+        elif isinstance(t, np.ndarray):
+            if not len(t) == x.shape[-2]:
+                raise ValueError("Length of t should match x.shape[-2].")
+            if not np.all(t[:-1] < t[1:]):
+                raise ValueError("Values in t should be in strictly increasing order.")
+        else:
+            raise ValueError("t must be a scalar or array-like.")
+    return x
+
+
+def validate_control_variables(x, u, trim_last_point=False):
+    """Ensure that control variables u are compatible with the data x.
+
+    Trims last control variable timepoint if set to True
     """
-    if multiple_trajectories:
-        if not isinstance(x, Sequence):
-            raise ValueError("x must be a list when multiple_trajectories is True")
-        if not isinstance(u, Sequence):
-            raise ValueError("u must be a list when multiple_trajectories is True")
-        if len(x) != len(u):
-            raise ValueError(
-                "x and u must be lists of the same length when "
-                "multiple_trajectories is True"
-            )
-
-        u_arr = [_check_control_shape(xi, ui, trim_last_point) for xi, ui in zip(x, u)]
-
-        if return_array:
-            u_arr = np.vstack(u_arr)
-
-    else:
-        u_arr = _check_control_shape(x, u, trim_last_point)
-
-    return u_arr
-
-
-def _check_control_shape(x, u, trim_last_point):
-    """
-    Convert control variables u to np.array(dtype=float64) and compare
-    its shape against x. Assumes x is array-like.
-    """
-    try:
-        u = np.array(u, dtype="float64")
-    except TypeError as e:
-        raise e(
-            "control variables u could not be converted to np.ndarray(dtype=float64)"
+    if not isinstance(x, Sequence):
+        raise ValueError("x must be a list when multiple_trajectories is True")
+    if not isinstance(u, Sequence):
+        raise ValueError("u must be a list when multiple_trajectories is True")
+    if len(x) != len(u):
+        raise ValueError(
+            "x and u must be lists of the same length when "
+            "multiple_trajectories is True"
         )
-    if np.ndim(u) == 0:
-        u = u[np.newaxis]
-    if u.ndim == 1:
-        u = u.reshape(-1, 1)
-        if len(x) != u.shape[0]:
+
+    def _check_control_shape(x, u, trim_last_point):
+        """
+        Compare shape of control variable u against x.
+        """
+        if u.shape[u.ax_time] != x.shape[x.ax_time]:
             raise ValueError(
                 "control variables u must have same number of rows as x. "
                 "u has {} rows and x has {} rows".format(u.shape[0], len(x))
             )
-    if u.ndim != 2:
-        u = u.reshape(u.size // u.shape[-1], u.shape[-1])
-        if (x.size // x.shape[-1]) != u.shape[0]:
-            raise ValueError(
-                "control variables u must have same number of rows as x. "
-                "u has {} rows and x has {} rows".format(u.shape[0], x.shape[0])
-            )
-    return u[:-1] if trim_last_point else u
+        return u[:-1] if trim_last_point else u
+
+    u_arr = [_check_control_shape(xi, ui, trim_last_point) for xi, ui in zip(x, u)]
+
+    return u_arr
 
 
-def drop_nan_rows(x, x_dot):
-    x = x[~np.isnan(x_dot).any(axis=1)]
-    x_dot = x_dot[~np.isnan(x_dot).any(axis=1)]
-    return x, x_dot
-
-
-def drop_random_rows(
-    x,
-    x_dot,
-    n_subset,
-    replace,
-    feature_library,
-    pde_library_flag,
-    multiple_trajectories,
-):
-    num_trajectories = feature_library.num_trajectories
-
-    # Can't choose random n_subset points if data is from a PDE
-    # (and therefore is spatially local).
-    # Need to unfold it and just choose n_subset from the temporal slices
-    if pde_library_flag is not None:
-        if pde_library_flag == "WeakPDE":
-            # Weak form needs uniform, ascending grid, so cannot replace
-            replace = False
-            s = [slice(None, None)] * feature_library.spatiotemporal_grid.ndim
-            s[-2] = 0
-            s[-1] = slice(None, -1)
-            spatial_grid = feature_library.spatiotemporal_grid[tuple(s)]
-            s = [0] * feature_library.spatiotemporal_grid.ndim
-            s[-2] = slice(None)
-            s[-1] = -1
-            temporal_grid = feature_library.spatiotemporal_grid[tuple(s)]
-            num_time = len(temporal_grid)
-            dims = spatial_grid.shape[:-1]
-        else:
-            # new bagging for arbitrary samples
-            n_samples = x.shape[0]
-            spatial_grid = feature_library.spatial_grid
-            dims = spatial_grid.shape[:-1]
-            if len(dims) > 0:
-                num_time = n_samples // np.product(dims) // num_trajectories
-            else:
-                num_time = n_samples // num_trajectories
-
-        n_features = x.shape[1]
-        if n_subset > num_time:
-            n_subset = num_time
-        rand_inds = np.sort(choice(range(num_time), n_subset, replace=replace))
-
-        if len(dims) > 0:
-            x_shaped = np.reshape(
-                x, np.concatenate([dims, [num_time * num_trajectories], [n_features]])
-            )
-        else:
-            x_shaped = np.reshape(
-                x, np.concatenate([[num_time * num_trajectories], [n_features]])
-            )
-        s0 = [slice(dim) for dim in x_shaped.shape]
-
-        rand_inds_total = []
-        for i in range(num_trajectories):
-            rand_inds_total.append(rand_inds + num_time * i)
-        s0[len(dims)] = rand_inds_total
-
-        if len(dims) > 0:
-            x_new = np.reshape(
-                x_shaped[tuple(s0)],
-                (np.product(dims) * n_subset * num_trajectories, x.shape[1]),
-            )
-        else:
-            x_new = np.reshape(
-                x_shaped[tuple(s0)], (n_subset * num_trajectories, x.shape[1])
-            )
-
-        if pde_library_flag == "WeakPDE":
-            spatiotemporal_grid = feature_library.spatiotemporal_grid
-            s1 = [slice(None)] * len(spatiotemporal_grid.shape)
-            s1[-2] = rand_inds
-            new_spatiotemporal_grid = spatiotemporal_grid[tuple(s1)]
-            feature_library.spatiotemporal_grid = new_spatiotemporal_grid
-            feature_library._set_up_weights()
-            s0[len(dims)] = rand_inds
-            if multiple_trajectories:
-                x_dot_new = [
-                    feature_library.convert_u_dot_integral(xi[tuple(s0)])
-                    for xi in feature_library.old_x
-                ]
-                x_dot_new = np.vstack(x_dot_new)
-            else:
-                x_dot_new = feature_library.convert_u_dot_integral(
-                    feature_library.old_x[tuple(s0)]
-                )
-        else:
-            x_dot_shaped = np.reshape(
-                x_dot,
-                np.concatenate([dims, [num_time * num_trajectories], [n_features]]),
-            )
-            x_dot_new = np.reshape(
-                x_dot_shaped[tuple(s0)],
-                (np.product(dims) * n_subset * num_trajectories, x.shape[1]),
-            )
-    else:
-        # Choose random n_subset points to use
-        rand_inds = np.sort(choice(range(np.shape(x)[0]), n_subset, replace=replace))
-        x_new = x[rand_inds, :]
-        x_dot_new = x_dot[rand_inds, :]
-
-    return x_new, x_dot_new
+def drop_nan_samples(x, y):
+    """Drops samples from x and y where there is either has a nan value"""
+    x_non_sample_axes = tuple(ax for ax in range(x.ndim) if ax != x.ax_sample)
+    y_non_sample_axes = tuple(ax for ax in range(y.ndim) if ax != y.ax_sample)
+    x_good_samples = (~np.isnan(x)).any(axis=x_non_sample_axes)
+    y_good_samples = (~np.isnan(y)).any(axis=y_non_sample_axes)
+    good_sample_ind = np.nonzero(x_good_samples & y_good_samples)[0]
+    x = x.take(good_sample_ind, axis=x.ax_sample)
+    y = y.take(good_sample_ind, axis=y.ax_sample)
+    return x, y
 
 
 def reorder_constraints(c, n_features, output_order="row"):
