@@ -1,8 +1,8 @@
 """
 Unit tests for SINDy class.
 
-Note: all tests should be encapsulated in functions whose
-names start with "test_"
+Note: all tests should be encapsulated in functions with
+names starting with "test_"
 
 To run all tests for this package, navigate to the top-level
 directory and execute the following command:
@@ -34,8 +34,6 @@ from pysindy.optimizers import ConstrainedSR3
 from pysindy.optimizers import SR3
 from pysindy.optimizers import STLSQ
 
-# from pysindy.utils import convert_u_dot_integral
-
 
 def test_get_feature_names_len(data_lorenz):
     x, t = data_lorenz
@@ -312,7 +310,6 @@ def test_score_pde(data_1d_random_pde):
         derivative_order=4,
         spatial_grid=x,
         include_bias=True,
-        is_uniform=True,
     )
     model = SINDy(feature_library=pde_lib).fit(
         u,
@@ -335,7 +332,6 @@ def test_score_pde(data_1d_random_pde):
         derivative_order=4,
         spatiotemporal_grid=XT,
         include_bias=True,
-        is_uniform=False,
     )
     model = SINDy(feature_library=weak_lib).fit(
         u,
@@ -610,10 +606,10 @@ def test_fit_warn(data_lorenz, params, warning):
     model = SINDy(optimizer=STLSQ(**params))
 
     with pytest.warns(warning):
-        model.fit(x, t)
+        model.fit(x, t=t)
 
     with pytest.warns(None) as warn_record:
-        model.fit(x, t, quiet=True)
+        model.fit(x, t=t, quiet=True)
 
     while True:
         try:
@@ -621,6 +617,12 @@ def test_fit_warn(data_lorenz, params, warning):
         except AssertionError:
             break
 
+    while True:
+        try:
+            warn_record.pop(DeprecationWarning)
+        except AssertionError:
+            break
+
     assert len(warn_record) == 0
 
 
@@ -702,7 +704,6 @@ def test_ensemble_pdes(data_1d_random_pde):
         derivative_order=4,
         spatial_grid=spatial_grid,
         include_bias=True,
-        is_uniform=True,
     )
     model = SINDy(feature_library=pde_lib).fit(
         u, t, ensemble=True, n_models=10, n_subset=len(t) // 2
@@ -726,7 +727,6 @@ def test_ensemble_weak_pdes(data_1d_random_pde):
         derivative_order=4,
         spatiotemporal_grid=XT,
         include_bias=True,
-        is_uniform=False,
     )
     model = SINDy(feature_library=weak_lib).fit(
         u, t=t, ensemble=True, n_models=2, n_subset=len(t) // 2
@@ -757,7 +757,6 @@ def test_library_ensemble_pde(data_1d_random_pde):
         derivative_order=4,
         spatial_grid=spatial_grid,
         include_bias=True,
-        is_uniform=True,
     )
     model = SINDy(feature_library=pde_lib).fit(
         u, t=t, library_ensemble=True, n_models=10
@@ -781,7 +780,6 @@ def test_library_ensemble_weak_pde(data_1d_random_pde):
         derivative_order=4,
         spatiotemporal_grid=XT,
         include_bias=True,
-        is_uniform=False,
     )
     model = SINDy(feature_library=weak_lib).fit(
         u, t=t, library_ensemble=True, n_models=10
@@ -813,7 +811,6 @@ def test_both_ensemble_pde(data_1d_random_pde):
         derivative_order=4,
         spatial_grid=spatial_grid,
         include_bias=True,
-        is_uniform=True,
     )
     model = SINDy(feature_library=pde_lib).fit(
         u, t=t, ensemble=True, library_ensemble=True, n_models=2
@@ -837,7 +834,6 @@ def test_both_ensemble_weak_pde(data_1d_random_pde):
         derivative_order=4,
         spatiotemporal_grid=XT,
         include_bias=True,
-        is_uniform=False,
     )
     model = SINDy(feature_library=weak_lib).fit(
         u, t=t, ensemble=True, library_ensemble=True, n_models=2
@@ -907,7 +903,6 @@ def test_multiple_trajectories_and_ensemble(diffuse_multiple_trajectories):
         function_names=library_function_names,
         derivative_order=2,
         spatial_grid=x,
-        is_uniform=True,
     )
 
     X, T = np.meshgrid(x, t, indexing="ij")
@@ -919,7 +914,6 @@ def test_multiple_trajectories_and_ensemble(diffuse_multiple_trajectories):
         derivative_order=2,
         spatiotemporal_grid=XT,
         K=100,
-        is_uniform=False,
     )
 
     optimizer = STLSQ(threshold=0.1, alpha=1e-5, normalize_columns=False)
